import redis
import logging
from django.conf import settings
from django.shortcuts import render
from math import radians, cos, sin, asin, sqrt
from django.http import HttpResponse,JsonResponse
import random
import requests
import uuid
import ipdb
from gm_rpcd.all import bind,RPCDFaultException

from .unit import refresh_cache_locations
#from rpc.decorators import bind_context, bind

r = redis.Redis(host=settings.REDIS_HOST, port=settings.REDIS_PORT,db=settings.REDIS_DB,password=settings.REDIS_PASSWORD,decode_responses=True)

rpc_client =settings.RPC_INVOKER
# Create your views here.

@bind('geoserver/home/nearly_hospitals')
def nearly_hospitals(q_lat,q_lng,radis=500,unit="km",count=20):
	'''
		查询最近医院
		@q_lnt 经度
		@q_lat 纬度
		@radis 方圆多少公里 
	'''
	#查询的经纬度
	#q_lat,q_lng=40.001930265172,116.4871890387796
	#数据缓存时间,现在为24小时
	cache_key="hospital_location"
	
	if not r.exists(cache_key):
		refresh_cache_locations()

	result=r.georadius(cache_key,q_lng,q_lat,radis,unit,withdist=True,count=20)

	data={
		"result":True,
		"data":result,
		"message":""
	}
	return data

@bind('geoserver/home/refresh')
def refresh():
	'''刷新redis中数据'''
	result={
		"result":True,
		"data":[],
		"message":""
	}
	try:
		refresh_cache_locations()
	except Exception as e:
		result.update({"result":False,"message":str(e)})

	return result



@bind("geoserver/home/hospitals_distance")
def specify_hospitals_distance(q_lng,q_lat,hospital_ids):
	'''
		到指定医院距离排序
	'''
	cache_key="hospital_location"

	#锁定临时的key
	data=[]
	message=""
	result=True

	try:
		temp_key="hospital_location_temp"
		lock_key="hospital_location_lock"
		with r.lock(lock_key, blocking_timeout=15) as lock:
			r.geoadd(cache_key,float(q_lng),float(q_lat),temp_key)
			for item in hospital_ids:
				distance=r.geodist(cache_key,temp_key,item)

				if not distance:
					distance=0

				data.append({"id":item,"distance":distance})
		#排序
		data=sorted(data,key=lambda x:x.get("distance"))
	except Exception as e:
		result,message=False,str(e)

	data={
		"result":result,
		"data":data,
		"message":message
	}
	return data





def get_location_by_address(request,address):
    """由地址获获取经纬度坐标

    :param address: 目标地址 '西城区西什库大街8号(近地安门西大街)
    :return: 经纬度信息 {"lng": 116.306735, "lat": 39.982951}
    """
    QQ_MAP_KEY = random.choice(settings.QQ_MAP_KEYS)

    params = {
        'address': address,
        'key': QQ_MAP_KEY,
        'output': 'json',
    }
    url = 'http://apis.map.qq.com/ws/geocoder/v1'
    response = requests.get(url, params=params)
    result = response.json()
    if result['status'] == 0:
        return result['result']['location']
    else:
        return None
	
